/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2012 Jolla Ltd.
 * Contact: Pami Ketolainen <pami.ketolainen@jollamobile.com>
 */

var INLINE_EDIT_COL_MAP = {
    short_short_desc: 'summary',
    assigned_to_realname: 'assigned_to',
    bug_status: 'status',
    bug_severity: 'severity',
    short_desc: 'summary'
};


var inlineEditCancel = function(ev)
{
    var link = $(ev.currentTarget);
    _closeInlineEdit(link);
    ev.preventDefault();
};

var _closeInlineEdit = function(link)
{
    link.attr('title', 'Edit')
        .html('<i class="fas fa-edit"></i>')
        .off('click').on('click', inlineEditOpen);
    var row = link.parents('tr').eq(0);
    row.next('tr.editor_row').remove();
    row.next('tr.comment_row').remove();
};


var inlineEditOpen = function(ev)
{
    var link = $(ev.currentTarget);
    // REDHAT EXTENSION START 1395045
    link.off('click');
    // REDHAT EXTENSION END 1395045
    var row = link.parents('tr').eq(0);
    var bug = link.data('bug');
    if (bug == undefined) {
        var bugId = row.attr('id').slice(1);
        Bug.get(bugId, function(bug) {
            link.data('bug', bug);
            bug.updated(_inlineEditUpdate);
            _openInlineEdit(bug, link, row);});
    } else {
        _openInlineEdit(bug, link, row);
    }
    ev.preventDefault();
};

var _openInlineEdit = function(bug, link, row)
{
    link.attr('title', 'Cancel')
        .html('<i class="fa fa-times"></i>')
        .off('click')
        .on('click', inlineEditCancel);

    var myid = 'edit_' + row.attr('id');

    var colCount = row.find('td').size();
    var editRow = $('<tr class="editor_row" id="' +myid + '"></tr>');
    row.find('td').each(function() {
        var orig_cell = $(this);
        var name;
        $.each(INLINE_EDIT_COLUMNS, function(i, col) {
            if (orig_cell.hasClass('bz_' + col + '_column')) {
                name = INLINE_EDIT_COL_MAP[col] || col;
                return false;
            }
        });
        var cell = $('<td></td>');
        cell.addClass(orig_cell[0].className);
        editRow.append(cell);
        if (name) {
            var fname = name == 'actual_time' ? 'work_time' : name
            try {
                var field = bug.field(fname);
            } catch (e) {
                return;
            }
            if (field.immutable) return;
            orig_cell.data('name', name);
            var input = bug.createInput(field, false, true);
            if (['remaining_time', 'estimated_time', 'work_time'].indexOf(field.name) > -1){
                input.css('width', '4em');
            } else {
                input.css('width', '100%');
            }
            if (field.name == 'work_time') cell.append("+");
            cell.append(input);
        } else if (orig_cell.hasClass('button_column')) {
//            $('<a class="inline_edit" title="Save"><i class="fa fa-floppy-o"></i></a>')
//                .appendTo(cell)
//                .on('click', _inlineEditSave);
        }
    });

    row.after(editRow);

    commentRow = $('<tr class="comment_row"><th>Comment:</th></tr>');
    var cell = $('<td>').attr('colspan', colCount - 1);
    var comment = $('<textarea name="comment" rows="1" cols="80"></textarea>')
        .focus(function(ev) { $(ev.currentTarget).attr('rows', 5)})
        .blur(function(ev) { $(ev.currentTarget).attr('rows', 1)})

    cell.append(comment);
    commentRow.append(cell);
    editRow.after(commentRow);
    var product_id = 'product';

    var comp_id = "#" + myid + " #component";

    var _cb = function(){
        var my_td = $('#' + myid + ' td.button_column');
        $('<a class="inline_edit" title="Save"><i class="fas fa-save"></i></a>')
                .appendTo(my_td)
                .on('click', _inlineEditSave);
        if($("#" + product_id) && $(comp_id).length <= 0) {
            $("#" + product_id).selectize()[0].selectize.$control[0].childNodes[0].title = "To change the Product you must have the Component field displayed.";
        }
    };

    if($("#" + myid + " #product").length) {
        product_id = myid + " #product";
        init_products({onclick: 1, id: product_id, callback : _cb});
        if($(comp_id).length <= 0){
            $("#" + product_id).selectize()[0].selectize.disable();
        }
    }

    if($(comp_id).length) {
       if(product_id === myid + " #product") {
           init_components({ onclick: 1, "product_id": product_id, id: myid + " #component" });
           $("#" + product_id).on('change', function() {
               $(comp_id).clear_unselected();
           });
       } else {
           init_components({ onclick: 1, product: bug._data.product, id: myid + " #component" });
       }

       $(comp_id).on('change', function() {
            if(!$("#" + myid + " #reset_contacts").length){
                $(comp_id).next().after('&nbsp;<input title="Reset all contact details" type="checkbox" name="reset_contacts" checked>');
            }
        });
    }

    if(product_id === 'product'){
        _cb();
    }

    $("#" + myid + " select").each(function() {
        if(! $(this).hasClass('selectized')) {
            $(this).selectize({});
        }
    });

    if($("#" + myid + " #version").length) $("#" + myid + " #version").selectize()[0].selectize.setValue(bug._data.version);
};

var _inlineEditSave = function(ev)
{
    var row = $(ev.currentTarget).parents('tr').eq(0).prev("tr.bz_bugitem");
    var link = row.find('a.inline_edit');
    var bug = link.data('bug');
    var editRows = row.nextUntil('tr.bz_bugitem');
    editRows.find('td > *').filter(':input').each(function() {
        // Make sure all values are set
        var input = $(this);
        var name = input.attr('name');
        if (!name) return;
        var val = input.val();
        if ($(this).hasClass('selectized')) val = $(this).selectize()[0].selectize.getValue();
        bug.set(name, val);
    });
    bug.save().done(function() {
        // Update bug row when save is done
        row.addClass('changed');
        _closeInlineEdit(link);
    });
    ev.preventDefault();
};

var _inlineEditUpdate = function(bug, name, value)
{
    var row = $('table.bz_buglist tr#b'+bug.value('id'));
    row.find('td').not('.button_column').each(function() {
        var element = $(this);
        if(element.data('name') != name) return;
        name = name == 'work_time' ? 'actual_time' : name;
        try {
            var field = bug.field(name);
        } catch(e) {
            return;
        }
        if (field.multivalue)
            value = value.join(', ');
        if(value != undefined) element.find('span,a').pushStack(element).first().text(value);
    });
};

var inlineVote = function (ev) {
    var link = $(ev.currentTarget);
    var row = link.parents('tr').eq(0);
    var bugId = row.attr('id').slice(1);
    new Rpc('Voting', 'for_bugs', {
            bug_ids: [bugId]
        })
        .complete(function (cb) {
            $(ev)._onAjaxComplete();
        })
        .fail(function (error) {
            alertify.error(error.message);
        })
        .done(function (res) {
            alertify.success(res);
        });
    ev.preventDefault();
};

var initInlineEditor = function() {
    $('table.bz_buglist tr.bz_buglist_header').append('<th class="button_column">Actions </tr>');
    var rows = $('table.bz_buglist tr.bz_bugitem');
    rows.append('<td class="button_column"><a href="#" title="Edit" class="inline_edit"><i class="fas fa-edit"></i></a> <a href="#" title="Vote" class="inline_vote"><i class="fas fa-vote-yea"></i></a></td>');
    $('table.bz_buglist a.inline_edit').click(inlineEditOpen);
    $('table.bz_buglist a.inline_vote').click(inlineVote);
    $('tr.bz_time_summary_line').append('<td class="bz_total">');
};

$(initInlineEditor);
