#!/usr/bin/perl
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

use strict;
use warnings;
use 5.10.1;

use lib qw(. lib);

BEGIN {
  use Bugzilla;
  use Bugzilla::Constants;
  Bugzilla->extensions;
}

=head1 NAME

add_bugs_to_queue.pl - adds bugs to the Job Queue

=head1 SYNOPSIS

    cd <bugzilla/dir/>
    ./extensions/RuleEngine/add_bugs_to_queue.pl

=head1 DESCRIPTION

This script adds bugs matching periodic rules to the job queue.

It should be run in a cron job or similar timed process.

=cut

Bugzilla->usage_mode(USAGE_MODE_CMDLINE);

use Bugzilla::Extension::RuleEngine::Rule;
use Bugzilla::Extension::RuleEngine::Pages;

# Log in as the system user.
Bugzilla->set_user(Bugzilla::User->new(
  {name => Bugzilla->params->{system_user}}));

my %matched = ();

# We only want to match active rules that have the periodic flag
my %args = (isactive => 1, is_periodic => 1);

for my $rule (@{Bugzilla::Extension::RuleEngine::Rule->match_rules(\%args)}) {

  # Find matching bugs
  my $bug_ids = _get_matching_bugs_for_rule($rule->current->match, $rule);
  foreach my $bug_id (@$bug_ids) {

    # We only need to add a bug once
    next if $matched{$bug_id}++;

    # Add the bug to the Rules Engine job queue.
    Bugzilla->job_queue->insert(
      'rule_engine',
      {
        action      => 'update',
        delta_ts    => 1,           # Always going to be true
        do_periodic => 1,
        bug_ids     => [$bug_id],
      }
    );
  }
}

