use 5.10.1;
use strict;
use warnings;

use lib qw(. lib t);
use Test::More tests => 21;

use Capture::Tiny ':all';
use HTTP::Parser;
use JSON;

use Bugzilla;
use Bugzilla::CGI;
use Bugzilla::Constants;
use Bugzilla::Error;
use Bugzilla::WebService::Constants;
use Bugzilla::WebService::Server::JSONRPC;

# Some RPC tests that have to run after the BRE tests.

BEGIN {
  $ENV{PROJECT}         = 'tests';
  $ENV{REQUEST_METHOD}  = 'POST';
  $ENV{CONTENT_TYPE}    = 'application/json';
  $ENV{PERL_JSON_DEBUG} = 1;
}

Bugzilla->usage_mode(USAGE_MODE_JSON);
my $dbh = Bugzilla->dbh();

my $rpc_debug = 0;

my $cgi = Bugzilla->cgi;
$cgi->header('application/json');

my %args;

local @INC = (bz_locations()->{extensionsdir}, @INC);

my $token = undef;

# Login to get a token
my %login_args = (login => 'myadmin@example.net', password => 'DudeMcDude',);

my $res = run_rpc('User.login', \%login_args);
$token = $res->{result}->{token};
my $user_id = $res->{result}->{id};

$res = run_rpc('RuleGroup.list');

ok(exists($res->{result}->{rule_groups}) . 'list rule groups');

my $id = $res->{result}->{rule_groups}->[0]->{id};

%args = (ids => [$id],);

$res = run_rpc('RuleGroup.get', \%args);

ok(exists($res->{result}->{rule_groups}) . 'get rule groups');

my $rgname = 'RG1';
%args = (
  name        => $rgname,
  description => 'Testing RPC rule group creation',
  view_groups => ['editbugs'],
  edit_groups => ['admin'],
  user        => 'myadmin@example.net',
  isactive    => 1,
);

$res = run_rpc('RuleGroup.create', \%args);

ok(exists($res->{result}->{rule_group}), 'create rule group');

my $new_rgname = $rgname . ' FTW';
my $rid        = $res->{result}->{rule_group}->{id};

%args = (rule_groups => [{id => $rid, name => $new_rgname,}],);
$res  = run_rpc('RuleGroup.update', \%args);

ok(exists($res->{result}->{changes}), 'update rule groups');
ok($res->{result}->{changes}->[0]->{name}->[0] eq $rgname, 'old name matches');
ok($res->{result}->{changes}->[0]->{name}->[1] eq $new_rgname,
  'new name matches');

exit;

# Run a jsonrpc request
sub run_rpc {
  my ($method, $params) = @_;

  my %args = (
    method  => $method,
    version => "1.1",
    token   => $token,
    params  => ($params || {}),
  );

  $cgi->delete_all();
  $cgi->cgi_error("");
  $cgi->param('POSTDATA', to_json(\%args));
  my $server = new Bugzilla::WebService::Server::JSONRPC;
  my ($stdout, $stderr, @result)
    = capture { $server->dispatch(WS_DISPATCH)->handle() };
  ok($result[0], $method);
## Useful for debugging
  if ($rpc_debug) {
    diag("stdout");
    diag explain $stdout;
    diag("stderr");
    diag explain $stderr;
    diag("result");
    diag explain \@result;
  }

  my $parser = HTTP::Parser->new(response => 1);
  ok($parser->add("HTTP/1.1\n" . $stdout), 'Parse response');

  my $resp = from_json($parser->object()->decoded_content());
  ok(!exists($resp->{error}), "no errors for $method") || diag explain $resp;

  eval { $dbh->bz_rollback_transaction() if ($dbh->bz_in_transaction); };
  return ($resp);
}

